#include "Controller.h"
#include "EventReceiver.h"

#include <iostream>
#include <queue>

//Basic controller (W,A,S,D,Q,E) setting speed/angle directly.
void controller(Train *t, Viewer *v)
{
	if (v->receiver.IsKeyDown(irr::KEY_KEY_W))
		t->setMotorSpeedAll(10);
	else if (v->receiver.IsKeyDown(irr::KEY_KEY_S))
		t->setMotorSpeedAll(-10);
	else
		t->setMotorSpeedAll(0);

	if (v->receiver.IsKeyDown(irr::KEY_KEY_A))
		t->setJointAngle(0, 0, 0.5);
	else if (v->receiver.IsKeyDown(irr::KEY_KEY_D))
		t->setJointAngle(0, 0, -0.5);
	else
		t->setJointAngle(0, 0, 0);

	if (v->receiver.IsKeyDown(irr::KEY_KEY_Q)) {
		t->setJointAngle(0, 1, 0.5);
		t->setJointAngle(1, 1, 0.5);
		for (int i = 2; i < t->getNumCarts(); i++) {
			t->setJointAngle(i, 1, 0);
		}
	}
	else if (v->receiver.IsKeyDown(irr::KEY_KEY_E)) {
		t->setJointAngle(0, 1, -0.5);
		t->setJointAngle(1, 1, -0.5);
		for (int i = 2; i < t->getNumCarts(); i++) {
			t->setJointAngle(i, 1, 0);
		}
	}
	else {
		for (int i = 0; i < t->getNumCarts(); i++) {
			t->setJointAngle(i, 1, 0);
		}
	}

	if (v->receiver.IsKeyDown(irr::KEY_KEY_R))
		t->resetTrain();
}
/*------------------------------------
void controllerFTL(Train *t, Viewer *v){
  static queue<Event> ftl_events[5]; //Length equal to (#piko-elements - 1)
  static int isLeft=false, isRight=false, isUp=false, isDown=false, isReset=false;
  static double angle_h, angle_v;
  static double mk = 0.15; //Length between wagon midpoints
  // Set angles as defined in Train for each iteration
  
  for (int i=0; i < t->getNumJoints(); i++){
    t->setJointAngle(i,0,t->getJointAngleSetPoint(i,0)); //Update horisontal reference
    t->setJointAngle(i,1,t->getJointAngleSetPoint(i,1)); //Update vertical reference
  }
  if (v->receiver.IsKeyDown(irr::KEY_KEY_A)){
	  cout << "A presed" << endl;
    if (!isLeft){
      isLeft = true;
      angle_h += 0.1;
      //      cout << "Angle inc " << angle_h << endl;
      if (angle_h > 0.7){ angle_h = 0.7; }
      t->setJointAngle(0,0,angle_h);
      ftl_events[0].push(Event(t->getTravelledDistance()+mk,angle_h,angle_v));
    }
  }
  else{ isLeft = false; }
  if (v->receiver.IsKeyDown(irr::KEY_KEY_D)){
	  cout << "D presed" << endl;
    if (!isRight){
      isRight = true;
      angle_h -= 0.1; //Decreasing lifts PIKO
      //      cout << "Angle dec " << angle_h <<endl;
      if (angle_h < -0.7){ angle_h = -0.7; }
      t->setJointAngle(0,0,angle_h);
      ftl_events[0].push(Event(t->getTravelledDistance()+mk,angle_h,angle_v));
    }
  }
  else{ isRight = false; }
  if (v->receiver.IsKeyDown(irr::KEY_KEY_Q)){
	  cout << "Q presed" << endl;
    if (!isUp){
      isUp = true;
      angle_v += 0.1; //Increasing turns PIKO downwards
      //      cout << "Angle_v inc " << angle_v <<endl;
      if (angle_v > 0.5){ angle_v = 0.5; }
      t->setJointAngle(0,1,angle_v);
      ftl_events[0].push(Event(t->getTravelledDistance()+mk,angle_h,angle_v));
    }
  }
  else{ isUp = false; }
  if (v->receiver.IsKeyDown(irr::KEY_KEY_E)){
	  cout << "E presed" << endl;
    if (!isDown){
      isDown = true;
      angle_v -= 0.1;
      //cout << "Angle_v dec " << angle_v <<endl;
      if (angle_v < -0.5){ angle_v = -0.5; }
      t->setJointAngle(0,1,angle_v);
      ftl_events[0].push(Event(t->getTravelledDistance()+mk,angle_h,angle_v));
    }
  }
  else{ isDown = false; }
  if (v->receiver.IsKeyDown(irr::KEY_KEY_W)){
	cout << "W presed" << endl; 
	t->setMotorSpeedAll(10);
  }
  else if (v->receiver.IsKeyDown(irr::KEY_KEY_S)){
	cout << "S presed" << endl;   
	t->setMotorSpeedAll(-10);
  }
  else
    t->setMotorSpeedAll(0);
  if (v->receiver.IsKeyDown(irr::KEY_KEY_R))
  { 
    if(isReset==0)
    {
        cout << "R presed" << endl;
        isReset++;
        t->resetTrain();
    }else if(isReset>0) isReset++;
  }else isReset=false;
  //Extracts events from the queue at 'mk' intervals, propagating commands from the leader.
	for (int i=1; i < t->getNumCarts(); i++){
	    if (ftl_events[i-1].empty()) 
            continue;
	    if (t->getTravelledDistance() >= ftl_events[i-1].front().dist){
	        t->setJointAngle(i, 0, ftl_events[i-1].front().hor);
	        t->setJointAngle(i, 1, ftl_events[i-1].front().vert);
	        ftl_events[i-1].front().dist += mk;
	        
            if (i < t->getNumCarts()-1) 
                ftl_events[i].push(ftl_events[i-1].front());
	        ftl_events[i-1].pop();
        }
    }
    
}


//----------------------------------- */


//Controller with "Follow the leader"-algorithm
void controllerFTL(Train *t, Viewer *v){

    //TODO: add an event for instant propagation

  static queue<Event> ftl_events[5]; //Length equal to (#piko-elements - 1)
  static int isLeft=false, isRight=false, isUp=false, isDown=false, isReset=false, isF=false, event=false;
  static double angle_h, angle_v;
  static double mk = 0.15; //Length between wagon midpoints
  static double old_d = 0,ndist=0,odist=0,maxheight=0;
  double auxH;
  static unsigned long int history=0;

    history++;
  // Set angles as defined in Train for each iteration
  /*if(history%20==0)
      if(t->getTravelledDistance() - old_d > 5){
          cout << "BOOM" << endl;
          old_d = t->getTravelledDistance();
      }
      */
  auxH = t-> getHeight(0);
  if(auxH > maxheight) maxheight = auxH;
  ndist=t->difference();
  if(abs(ndist - odist)> 0.10)
  {
	  cout << ndist - odist<<":"<<history<<endl;
	  odist = ndist;
  }

  for (int i=0; i < t->getNumJoints(); i++){
    t->setJointAngle(i,0,t->getJointAngleSetPoint(i,0)); //Update horisontal reference
    t->setJointAngle(i,1,t->getJointAngleSetPoint(i,1)); //Update vertical reference
  }

  if (v->receiver.IsKeyDown(irr::KEY_KEY_A)){
	  
    if (!isLeft){
        cout << "A pressed" << endl;
      event=true;
      isLeft = true;
      angle_h += 0.1;
      //      cout << "Angle inc " << angle_h << endl;
      if (angle_h > 0.7){ angle_h = 0.7; }
      t->setJointAngle(0,0,angle_h);
      ftl_events[0].push(Event(t->getTravelledDistance()+mk,angle_h,angle_v));
      isLeft = true;
    }
  }
  else{ isLeft = false; }

  if (v->receiver.IsKeyDown(irr::KEY_KEY_D)){
	  
    if (!isRight){
      event=true;
      cout << "D pressed" << endl;
      isRight = true;
      angle_h -= 0.1; //Decreasing lifts PIKO
      //      cout << "Angle dec " << angle_h <<endl;
      if (angle_h < -0.7){ angle_h = -0.7; }
      t->setJointAngle(0,0,angle_h);
      ftl_events[0].push(Event(t->getTravelledDistance()+mk,angle_h,angle_v));
      isRight = true;
    }
  }
  else{ isRight = false; }

    if (v->receiver.IsKeyDown(irr::KEY_KEY_Q)){
        if (!isUp){
            event=true;
            //cout << "Q pressed" << endl;
            isUp = true;
            angle_v += 0.1; //Increasing turns PIKO downwards
            //      cout << "Angle_v inc " << angle_v <<endl;
            if (angle_v > 1.3){ angle_v = 1.3; }
            t->setJointAngle(0,1,angle_v);
            //if( angle_v < 0.5 )
                ftl_events[0].push(Event(t->getTravelledDistance()+mk,angle_h,angle_v));

            //cout << "Angle_v dec " << angle_v <<"::" <<t->getJointAngle(0,1) <<endl;
            isUp = true;
        }
    }else{ isUp = false; }
  
    if (v->receiver.IsKeyDown(irr::KEY_KEY_E))
    {
        if (!isDown)
        {
            event=true;
            cout << "E pressed" << endl;
            isDown = true;
            angle_v -= 0.1;
            //cout << "Angle_v dec " << angle_v <<endl;
            if (angle_v < -1.3){ angle_v = -1.3; }
            t->setJointAngle(0,1,angle_v);
            //if(angle_v < -0.5)
                ftl_events[0].push(Event(t->getTravelledDistance()+mk,angle_h,angle_v));
            cout << "Angle_v dec " << angle_v <<"::" <<t->getJointAngle(0,1) <<endl;
            isDown = true;
        }
    }else{ isDown = false; }

  //Forward and reverse movement
  if (v->receiver.IsKeyDown(irr::KEY_KEY_W)){
	//cout << "W pressed" << endl; 
	event=true;
	t->setMotorSpeedAll(10);
  }else if (v->receiver.IsKeyDown(irr::KEY_KEY_S)){
	//cout << "S pressed" << endl;   
	event=true;
	t->setMotorSpeedAll(-10);
  }else
    t->setMotorSpeedAll(0);

  if (v->receiver.IsKeyDown(irr::KEY_KEY_R))
  { 
    if(!isReset)
    {
        event=true;
        cout << "R pressed" << endl;
        for(int i=0; i< t->getNumCarts() -1 ; i++){
            while(ftl_events[i].size()>0){
                ftl_events[i].pop();
            }
        }
        isReset++;
        t->resetTrain();
        angle_h = 0;
        angle_v = 0;
        isReset = true;
    }
  }else isReset=false;

    if (v->receiver.IsKeyDown(irr::KEY_KEY_F))
    {
        if(!isF){
            cout << "          -----\\F/----          " << endl;
            cout << "Joint 0: V=" << t->getJointAngle(0,1) << " H=" << t->getJointAngle(0,0) <<endl;
            cout << "Joint 1: V=" << t->getJointAngle(1,1) << " H=" << t->getJointAngle(1,0) <<endl;
            cout << "Joint 2: V=" << t->getJointAngle(2,1) << " H=" << t->getJointAngle(2,0) <<endl;
            cout << "Joint 3: V=" << t->getJointAngle(3,1) << " H=" << t->getJointAngle(3,0) <<endl;
            cout << "Joint 4: V=" << t->getJointAngle(4,1) << " H=" << t->getJointAngle(4,0) <<endl << endl;

            cout << "Travelled distance= " << t->getTravelledDistance() << endl<<endl;
            cout << "Height leader= " << t-> getHeight(0) << endl<<endl;
            cout << "Max. Height=" << maxheight << endl;
            cout << "Event queue" << endl;
            cout << "Cart 0 (leader): #events = " << ftl_events[0].size() << endl;
            cout << "Cart 1         : #events = " << ftl_events[1].size() << endl;       
            cout << "Cart 2         : #events = " << ftl_events[2].size() << endl;       
            cout << "Cart 3         : #events = " << ftl_events[3].size() << endl;       
            cout << "Cart 4         : #events = " << ftl_events[4].size() << endl;        
            
            cout << "          -----/F\\----          " << endl;
            isF = true;
        }
    }else isF = false; 

	if (v->receiver.IsKeyDown(irr::KEY_KEY_I))
	{
		for (int i=1; i < t->getNumCarts(); i++){ //For all the carts
			if (ftl_events[i-1].empty()) // If the previous cart's event queue is empty, continue
				continue;

			// If the train's travelled distance is greater or equal than the previous cart's distance
			t->setJointAngle(i, 0, ftl_events[i-1].front().hor);    // Set the joint's H-angle to be the previous cart's first event's h-angle
			t->setJointAngle(i, 1, ftl_events[i-1].front().vert);   // Set the joint's V-angle to be the previous cart's first event's v-angle
			ftl_events[i-1].front().dist += mk;                     // Increment the previous cart's front  event's distance by "mk"

			if (i < t->getNumCarts()-1)                             // If there are more carts
				ftl_events[i].push(ftl_events[i-1].front());        // Push the previous cart's events to the current cart's event queue

			ftl_events[i-1].pop();                                  // Pop the previous cart's event queue
			//cout << "POP<" << history << ">: " << i-1 << endl;

		}
	}
	if(event)
	{
		for (int i=1; i < t->getNumCarts(); i++){ //For all the carts
			if (ftl_events[i-1].empty()) // If the previous cart's event queue is empty, continue
				continue;
			if (t->getTravelledDistance() >= ftl_events[i-1].front().dist){
			// If the train's travelled distance is greater or equal than the previous cart's distance
				t->setJointAngle(i, 0, ftl_events[i-1].front().hor);    // Set the joint's H-angle to be the previous cart's first event's h-angle
				t->setJointAngle(i, 1, ftl_events[i-1].front().vert);   // Set the joint's V-angle to be the previous cart's first event's v-angle
				ftl_events[i-1].front().dist += mk;                     // Increment the previous cart's front  event's distance by "mk"

				if (i < t->getNumCarts()-1)                             // If there are more carts
					ftl_events[i].push(ftl_events[i-1].front());        // Push the previous cart's events to the current cart's event queue
				ftl_events[i-1].pop();                                  // Pop the previous cart's event queue
				//cout << "POP<" << history << ">: " << i-1 << endl;
			}
		}
	}
}

void controllerR(Train *t, Viewer *v){

    //TODO: add an event for instant propagation

  static queue<Event> ftl_events[5]; //Length equal to (#piko-elements - 1)
  static int isLeft=false, isRight=false, isUp=false, isDown=false, isReset=false, isF=false, event=false;
  static double angle_h, angle_v;
  static double mk = 0.15; //Length between wagon midpoints
  static double old_d = 0,ndist=0,odist=0,maxheight=0;
  double auxH;
  static unsigned long int history=0;
  static int forward=1;

    history++;
  // Set angles as defined in Train for each iteration
  /*if(history%20==0)
      if(t->getTravelledDistance() - old_d > 5){
          cout << "BOOM" << endl;
          old_d = t->getTravelledDistance();
      }
      */
  auxH = t-> getHeight(0);
  if(auxH > maxheight) maxheight = auxH;
  ndist=t->difference();
  if(abs(ndist - odist)> 0.10)
  {
	  cout << ndist - odist<<":"<<history<<endl;
	  odist = ndist;
  }

  for (int i=0; i < t->getNumJoints(); i++){
    t->setJointAngle(i,0,t->getJointAngleSetPoint(i,0)); //Update horisontal reference
    t->setJointAngle(i,1,t->getJointAngleSetPoint(i,1)); //Update vertical reference
  }

  forward++;
  //Forward and reverse movement
  if (forward > 0 && forward < 1000){
	//cout << "W pressed" << endl; 
	event=true;
    t->setMotorSpeedAll(10);
    if( forward%10 == 0)
    {    angle_v += 0.1; //Increasing turns PIKO downwards
        //      cout << "Angle_v inc " << angle_v <<endl;
        if (angle_v > 1.3){ angle_v = 1.3; }
        t->setJointAngle(0,1,angle_v);
        //if( angle_v < 0.5 )
            ftl_events[0].push(Event(t->getTravelledDistance()+mk,angle_h,angle_v));
    }
  }

  if (forward == 1000)
  { 
    event=true;
    cout << "R pressed" << endl;
    for(int i=0; i< t->getNumCarts() -1 ; i++){
        while(ftl_events[i].size()>0){
            ftl_events[i].pop();
        }
    }
    isReset++;
    t->resetTrain();
    angle_h = 0;
    angle_v = 0;
    isReset = true;
    forward = -250;
  }

    if (v->receiver.IsKeyDown(irr::KEY_KEY_F))
    {
        if(!isF){
            cout << "          -----\\F/----          " << endl;
            cout << "Joint 0: V=" << t->getJointAngle(0,1) << " H=" << t->getJointAngle(0,0) <<endl;
            cout << "Joint 1: V=" << t->getJointAngle(1,1) << " H=" << t->getJointAngle(1,0) <<endl;
            cout << "Joint 2: V=" << t->getJointAngle(2,1) << " H=" << t->getJointAngle(2,0) <<endl;
            cout << "Joint 3: V=" << t->getJointAngle(3,1) << " H=" << t->getJointAngle(3,0) <<endl;
            cout << "Joint 4: V=" << t->getJointAngle(4,1) << " H=" << t->getJointAngle(4,0) <<endl << endl;

            cout << "Travelled distance= " << t->getTravelledDistance() << endl<<endl;
            cout << "Height leader= " << t-> getHeight(0) << endl<<endl;
            cout << "Max. Height=" << maxheight << endl;
            cout << "Event queue" << endl;
            cout << "Cart 0 (leader): #events = " << ftl_events[0].size() << endl;
            cout << "Cart 1         : #events = " << ftl_events[1].size() << endl;       
            cout << "Cart 2         : #events = " << ftl_events[2].size() << endl;       
            cout << "Cart 3         : #events = " << ftl_events[3].size() << endl;       
            cout << "Cart 4         : #events = " << ftl_events[4].size() << endl;        
            
            cout << "          -----/F\\----          " << endl;
            isF = true;
        }
    }else isF = false; 

if (v->receiver.IsKeyDown(irr::KEY_KEY_I))
{
    for (int i=1; i < t->getNumCarts(); i++){ //For all the carts
        if (ftl_events[i-1].empty()) // If the previous cart's event queue is empty, continue
            continue;

        // If the train's travelled distance is greater or equal than the previous cart's distance
        t->setJointAngle(i, 0, ftl_events[i-1].front().hor);    // Set the joint's H-angle to be the previous cart's first event's h-angle
        t->setJointAngle(i, 1, ftl_events[i-1].front().vert);   // Set the joint's V-angle to be the previous cart's first event's v-angle
        ftl_events[i-1].front().dist += mk;                     // Increment the previous cart's front  event's distance by "mk" 

        if (i < t->getNumCarts()-1)                             // If there are more carts 
            ftl_events[i].push(ftl_events[i-1].front());        // Push the previous cart's events to the current cart's event queue
        
        ftl_events[i-1].pop();                                  // Pop the previous cart's event queue
        //cout << "POP<" << history << ">: " << i-1 << endl;

    }
}

  //Extracts events from the queue at 'mk' intervals, propagating commands from the leader.
   if(event)
	for (int i=1; i < t->getNumCarts(); i++){ //For all the carts
	    if (ftl_events[i-1].empty()) // If the previous cart's event queue is empty, continue
            continue;
	    if (t->getTravelledDistance() >= ftl_events[i-1].front().dist){ 
        // If the train's travelled distance is greater or equal than the previous cart's distance
	        t->setJointAngle(i, 0, ftl_events[i-1].front().hor);    // Set the joint's H-angle to be the previous cart's first event's h-angle
	        t->setJointAngle(i, 1, ftl_events[i-1].front().vert);   // Set the joint's V-angle to be the previous cart's first event's v-angle
	        ftl_events[i-1].front().dist += mk;                     // Increment the previous cart's front  event's distance by "mk" 
	        
            if (i < t->getNumCarts()-1)                             // If there are more carts 
                ftl_events[i].push(ftl_events[i-1].front());        // Push the previous cart's events to the current cart's event queue
	        ftl_events[i-1].pop();                                  // Pop the previous cart's event queue
	        //cout << "POP<" << history << ">: " << i-1 << endl;
        }
    }
}

void controllerUp(Train *t, Viewer *v){
	static int iteration = 0;

	t->setMotorSpeedAll(0);
	for (int i = 0; i < t->getNumJoints(); i++)
		t->setJointAngle(i, 0, 0);
    
	int joint = (iteration/100<=t->getNumJoints())?iteration/100:0;
	if (iteration < 600) {
        //cout << "600:" << iteration << endl;
		for (int i = 0; i < joint; i++)
			t->setJointAngle(i, 1, 0.55);
		for (int i = joint; i < t->getNumJoints(); i++)
			t->setJointAngle(i, 1, 0);
	} else if ((iteration > 50) && (iteration < 100)) { //Never enters
        cout << "50-100" << endl;
		for (int i = 0; i < t->getNumJoints(); i++)
			t->setJointAngle(i, 1, 0.5);
	} else if ((iteration > 100) && (iteration < 200)) { //Never enters
        cout << "100-200" << endl;
		for (int i = 0; i < t->getNumJoints(); i++)
			t->setJointAngle(i, 1, 0.5);
	} else if (iteration > 200) {
        //cout << "200:" << iteration << endl;
	}

	iteration++;

    if (v->receiver.IsKeyDown(irr::KEY_KEY_F))
    {
        //if(!isF){
            cout << "          -----\\F/----          " << endl;
            cout << "Joint 0: V=" << t->getJointAngle(0,1) << " H=" << t->getJointAngle(0,0) <<endl;
            cout << "Joint 1: V=" << t->getJointAngle(1,1) << " H=" << t->getJointAngle(1,0) <<endl;
            cout << "Joint 2: V=" << t->getJointAngle(2,1) << " H=" << t->getJointAngle(2,0) <<endl;
            cout << "Joint 3: V=" << t->getJointAngle(3,1) << " H=" << t->getJointAngle(3,0) <<endl;
            cout << "Joint 4: V=" << t->getJointAngle(4,1) << " H=" << t->getJointAngle(4,0) <<endl << endl;

            cout << "Travelled distance= " << t->getTravelledDistance() << endl<<endl;

          /*  cout << "Event queue" << endl;
            cout << "Cart 0 (leader): #events = " << ftl_events[0].size() << endl;
            cout << "Cart 1         : #events = " << ftl_events[1].size() << endl;       
            cout << "Cart 2         : #events = " << ftl_events[2].size() << endl;       
            cout << "Cart 3         : #events = " << ftl_events[3].size() << endl;       
            cout << "Cart 4         : #events = " << ftl_events[4].size() << endl;        
           */ 
            cout << "          -----/F\\----          " << endl;
        //    isF = true;
        //}
    }//else isF = false;
}

void controllerS(Train *t, Viewer *v){
    static unsigned long int sHist=0;
    t->setMotorSpeedAll(0);
	for (int i = 0; i < t->getNumJoints(); i++)
		t->setJointAngle(i, 0, 0);


	if( sHist >= 25 && sHist < 100)
	{
		t->setJointAngle(0, 1, 1.2);
    	t->setJointAngle(1, 1, 1.2);
    	t->setJointAngle(2, 1, 0);
    	t->setJointAngle(3, 1, 0);
	}
	if( sHist >= 150 && sHist < 200)
	{
		t->setJointAngle(0, 1, 1.2);
		t->setJointAngle(1, 1, 1.2);
		t->setJointAngle(2, 1, 0);
		t->setJointAngle(3, 1, 0);
	}
	if( sHist >= 250 && sHist < 300)
	{
		t->setJointAngle(0, 1, 1.2);
		t->setJointAngle(1, 1, 1.2);
		t->setJointAngle(2, 1, 0);
		t->setJointAngle(3, 1, 0);
	}
	if( sHist >= 300 && sHist < 350)
	{
		t->setJointAngle(0, 1, 1.2);
		t->setJointAngle(1, 1, 1.2);
		t->setJointAngle(2, 1, 1.2);
		t->setJointAngle(3, 1, 0);
	}
	if( sHist >= 350 && sHist < 400)
	{
		t->setJointAngle(0, 1, 0.3);
				t->setJointAngle(1, 1, 1.3);
				t->setJointAngle(2, 1, 1.2);
				t->setJointAngle(3, 1, 0.0);
	}
	if( sHist >= 400 && sHist < 450 )
		{
			t->setJointAngle(0, 1, 0.0);
					t->setJointAngle(1, 1, 1.3);
					t->setJointAngle(2, 1, 1.2);
					t->setJointAngle(3, 1, 0.0);
		}
	if( sHist >= 450 )
	{
		t->setJointAngle(0, 1, 1.0);
		t->setJointAngle(1, 1, 1.3);
		t->setJointAngle(2, 1, 1.2);
		t->setJointAngle(3, 1, -0.6);
	}

    /*if (sHist >= 25 && sHist < 100){
    	t->setJointAngle(1, 1, 1.3);
    	t->setJointAngle(3, 1, 1.3);
    }
    if(sHist >= 100 && sHist < 250){
    	t->setJointAngle(1, 1, 1.3);
    	t->setJointAngle(0, 1, 0.6);
    }
    if(sHist >= 250 && sHist < 500){
        	t->setJointAngle(2, 1, 1.1);
        	t->setJointAngle(3, 1, -1.3);
        }
    */




    /*if(sHist >= 150 && sHist < 350){
    	t->setJointAngle(0, 1, -0.6);
    }
    if(sHist >= 250 ){
    	//t->setJointAngle(3, 1, -1.0);
    }
    if(sHist >= 350 ){
        	t->setJointAngle(0, 1, 0);
    }
    if(sHist >= 400 && sHist >= 450 ){
        	t->setJointAngle(1, 1, 1.1);
    }
    if(sHist >= 450 && sHist >= 500 ){
        	t->setJointAngle(3, 1, -1.3);
    }if(sHist >= 500 && sHist >= 550 ){
    	t->setJointAngle(0, 1, -1.3);
    	t->setJointAngle(1, 1, 1.3);
    }
    */
    //cout << t->getJointAngle(0,1) << endl;
    sHist++;
}
