/*
* Requirements:
- Data strustures needed for the FTL:
-- eventqeues
- Action selection {forward, backwards, up, down}
- CACLA algorithm:
-- V_(t+1)(S_t) <- Beta_t- r_t + gamma*V_t(S_(t+1)
-- 1. Initialize A_0(s), for all s
-- 2. Initialize V_o(s), for all s
-- 3. Select s_0
-- 4. For each step t=0,1,2...:
--- 5. Select a_t with exploration Around A_t(s_t) (e.g. Gaussian exploration)
--- 6. Perform a_t, observe r_t, s_(t+1)
--- 7. If s_(t+1) is terminal:
---- 8. V_(t+1) (s_t) <--Beta_t-- r_t
---- 9. Select new s_(t+1) (starting point for next episode)
--- 10. Else:
---- 11. V_(t+1)(s_t) <--Beta_t-- r_t + Gamma*V_t(s_(t+1)
--- 12. If V_(t+1)(s_t) > V_t(s_t):
---- 13. A_(t+1)(s_t) <-- alfa_t -- a_t

- Defining the problem:
-- A state is given by the first modules' position and the angle of the first joint
-- Each feature has a width of about the half of the distance between two modules.
-- Phi_s(i) = exp( - ( ||s - c_i||^2 )/(2*(Sigma_i)^2 )
--- Sigma is the widht of the feature (starting with half of distance between carts ca. 0.06m)
*/


#include "RL.h"

using namespace std;

RL::RL(int _id, double valRate, double polRate, double rew){
    id=_id;

    //ANN network(1,10,5,H_FEATURES,V_FEATURES);
    srand (time(NULL));
    myfile.open ("heights.txt", ios::out);

    beta = valRate; //Critic's learning rate
    alfa = polRate; //Actor's learning rate
    reward = rew;
    maxHeight = 0;
    gamma = 0.1;
    speed = 0;
    step = 0;
    pstep = -1;
    oldHeight = 0;
    milestone = 0.2;
    activeJoints = 3;
    Vt = -1000;
    rate = 0.5;

    //ANN(int nInputs, int nFeatures, int nOutputs, int nrH, int nrV, double x0, double y0, double d, double firstC );
    network = new ANN(2,H_FEATURES*V_FEATURES,5,H_FEATURES,V_FEATURES, 0, 0, DIST,0);

    for(int i =0; i < 4; i++)
    {
    	angles[i]=0.0;
    }

}

void RL::initialize()
{
    network->restoreWeights();
    cout << "Initialized" << endl;
}

void RL::preStep(Train *t, bool keep)
{
	double a;
    int ac;

    if (!keep)
    	pstep++;
	//double xp;
	// Select action
	/*
	if( milestone > 0.2)
	{
		activeJoint = 2;
	}*/
	oldpos[0]=t->getX(0);
	oldpos[1]=t->getY(0);
	oldpos[2]=t->getHeight(0);

	oldHeight = t->getHeight(0);

	network->computeANN(oldpos,activeJoints);
	Vt = network->getValue();
	a = network->getPolicy(pstep%activeJoints);

    actionAngle(t, pstep%activeJoints, a, keep);
//    cout << "out prestep" << endl;

    //cout << "out of prestep:" << pstep << endl;
}
void RL::next(Train *t)
{
    double a;
    int joint;
    double pos[3];

    pos[0] = t->getX(0);
    pos[1] = t->getY(0);
    pos[2] = t->getHeight(0);
	network->computeANN(pos,activeJoints);
    
    pstep++;
    joint = pstep%activeJoints;

	a = network->getPolicy(joint);

	actionAngle(t, joint, a, false);

	refreshAngles(t);
}

void RL::actionAngle(Train *t,int joint, double a, bool keep)
{
	static double angle_max = -10;
	//double a1 = gaussianGenerator(greedy1,0.1);
	double angle=-100;

	if(!keep)
	{
		angle = gaussianGenerator(a,rate);

		if(angle > 1.0)
			angle = 1.0;
		else if( angle < -1.0)
			angle = -1.0;

//		cout << "angle=" << angle << endl;
		if(angle > angle_max)
			angle_max=angle;
		//cout << "mean:"<<a << " angle="<<angle<< " rate=" << rate << endl;
		if(pstep%10000 == 0 ){
			if( rate > 0.05)
			{
				rate= rate*0.99;
			}else{
				rate = 0.05;
			}
			cout << "new rate="<< rate << endl;
			cout << "angle_max=" << angle_max << endl;
		}
		angles[joint]= angle;
		if (angles[joint] < -1.3){ angles[joint] = -1.3; }
			t->setJointAngle(joint,1,angles[joint]);
		action1 = angle;
	}

	refreshAngles(t);
}

int RL::postStep(Train *t)
{
    double tdE, rt, auxVal;
    double pos[3];
    double Vt_1, h;
    double greedy1, greedy2, aux;
    int ret=0;
    int x,z;
    double xp;
    double npol;

    pos[0] = t->getX(0);
    pos[1] = t->getY(0);
    pos[2] = t->getHeight(0);

    h = pos[2];

//    cout << "start poststep:" << pstep << endl;
    network->computeANN(pos,activeJoints);
    Vt_1 = network->getValue();
    aux = network->horizontalDistance(pos);
    //cout << "Policy="<< action1 << " Bounds=" << aux << endl;
    //cout << "poststep:" << pstep << endl;
//    ofstream file;
//    file.open("info.txt", ios::out|ios::app);

    //file << "h:" << h << "   oldHeight:"<< oldHeight << endl;
    rt= h - oldHeight;

    if( h > 2.0)
    	rt = -100;

    if(abs(aux) > 1.0 )
	{
		cout << "Out of bounds["<<pos[0]<<","<< pos[1] <<"]:"<< aux << endl;
		actions(5,t);
		rt = -10;
		ret = -1;
	}


    tdE = rt + gamma*Vt_1-Vt;
//    file << "tdE="<<tdE<<" rt=" << rt << " gamma="<< gamma << " Vt_1=" <<Vt_1 << " Vt="<<Vt <<endl;
    //tdE = t->getHeight(0) - oldHeight;

    if(tdE > 0) // New state is better
    {
    	// If the action taken was different from the one considered optimal
//    	cout << "Updating policy towards "<< action1*alfa << endl;
//    	cout << "tdE="<<tdE<<" rt=" << rt << " gamma="<< gamma << " Vt_1=" <<Vt_1 << " Vt="<<Vt <<endl;
    	network->updateWeights(pstep%activeJoints,action1,alfa );
    	//cout << "updating towards "<< action1 << endl;
	}
    // Update critic
    Vt_1 = Vt + beta*tdE;

    //file << "["<< pos[0] << ","<< pos[2] <<"] \tVt_1="<<Vt_1<<" Vt=" << Vt << " beta:" << beta <<" tde=" << tdE << " rt="<<rt <<" action=" <<action1 << " Expl=" << exploratory[pstep%activeJoints] <<endl << endl;
//    file.close();
    //cout << "Critic updated with "<< Vt_1 << endl;

    //cout << "["<< pos[0] << ","<< pos[2] <<"]" <<"Error="<< errorV  <<" Height=" << oldHeight << " Value=" << network->getValue() << endl;
    double newval = network->updateWeights(network->outputs-1,Vt_1, 1);
//    cout << "Critic was "<<Vt <<"\tis now=" << newval << " tdE="<<tdE<< endl;

    //Update oldpos with the current position
    step++;

//    cout << "end poststep:" << pstep << endl;
    return ret;
}


void RL::nextActions6(Train *t, double greedy1, int joint)
{

	//New structure for keeping angles necessary to attain the desired angles. Requires refreshing all the joints after each step
	// TODO: ^
    static double rate=0.6;
	//double a1 = gaussianGenerator(greedy1,0.1);
    double r;

    //r = randDouble(0,1);
    r = rand();
    r = r/RAND_MAX;


    if(pstep == 40000){
    	rate = 0.95;
    	cout << ">>>>>>>>>>>>>>>>>>>"<< endl <<"new rate=" << endl << rate << "<<<<<<<<<<<<<<<<<<<" <<endl;
    }else if(pstep == 20000){
    	rate = 0.90;
    	cout << ">>>>>>>>>>>>>>>>>>>"<< endl <<"new rate=" << endl << rate << "<<<<<<<<<<<<<<<<<<<" <<endl;
    }

    if( r <= rate ) //Greedy
    {
    	greedy++;
        exploratory[pstep%activeJoints]=false;
        if(greedy1 < 0)
	    {
		    angles[joint]-=0.01;
		    if (angles[joint] < -1.3){ angles[joint] = -1.3; }
		    t->setJointAngle(joint,1,angles[joint]);
            action1 = -0.1-joint ;
	    }else if(greedy1 > 0)
	    {
		    angles[joint]+=0.01;
		    if (angles[joint] > 1.3){ angles[joint] = 1.3; }
		    t->setJointAngle(joint,1,angles[joint]);
            action1 = 0.1+joint;
        }else
        	action1 = 0;
    }else   //Exploring
    {
    	expl++;
    	//cout << "exploring:"<<e<<"::"<<g;
        exploratory[pstep%activeJoints]=true;
        if(greedy1 < 0)
	    {
        	//cout << " increasing" << endl;
		    angles[joint]+=0.01;
		    if (angles[joint] > 1.3){ angles[joint] = 1.3; }
		    t->setJointAngle(joint,1,angles[joint]);
            action1 = 0.2+joint;
	    }else if(greedy1 > 0)
	    {
	    	//cout << "decreasing" << endl;
		    angles[joint]-=0.01;
		    if (angles[joint] < -1.3){ angles[joint] = -1.3; }
		    t->setJointAngle(joint,1,angles[joint]);
            action1 = -0.2-joint;
        }else{
        	int a;
        	a = rand();
        	if(a%2==0){
        		//cout << " increasing" << endl;
				angles[joint]+=0.01;
				if (angles[joint] > 1.3){ angles[joint] = 1.3; }
				t->setJointAngle(joint,1,angles[joint]);
				action1 = 0.3+joint;
        	}else{
        		angles[joint]-=0.01;
				if (angles[joint] < -1.3){ angles[joint] = -1.3; }
				t->setJointAngle(joint,1,angles[joint]);
				action1 = -0.3-joint;
        	}
        }

    }

	refreshAngles(t);
	ofstream file;

	//file.open("randoms.txt", ios::out|ios::app);
	//file << "Greedy="<<g<<" Exploratory="<<e << endl;
	file.close();
}

void RL::refreshAngles(Train *t)
{
	for(int i=0; i<= t->getNumJoints();i++)
	{
		t->setJointAngle(i,1,angles[i]);
	}
}
void RL::nextActions(Train *t, double greedy1, double greedy2)
{
    double a1 = gaussianGenerator(greedy1,0.001);
    double a2 = gaussianGenerator(greedy2,0.001);
    static int g=0, h=0;


    if( greedy1 > -90.0)
    {
		if(a1<0 ) //Decrease angle
		{
			actions(1, t);
			action1 = a1;
			//cout << "Decrease angle" << endl;
		}else if(a1>0)
		{
			actions(2, t);
			action1 = a1;
			//cout << "Increase angle" << endl;
		}else{
				actions(6, t);
				action1 = 0;
		}
		//cout << "action1="<< action1 << endl;
	}else{
		action1 = -100;
	}

    if( greedy2 > -90.0)
    {
		if(a2<0)
		{
			actions(3, t);
			action2 = a2;
			//cout << "Decrease speed" << endl;
		}else if(a2>0)
		{
			actions(4, t);
			action2 = a2;
			//cout << "Decrease increase speed" << endl;
		}else{
			actions(6, t);
			action2 = 0;
		}
	}else{
		action2 = -100;
	}
    refreshAngles(t);
}


void RL::actions(int action, Train *t)
{
    static queue<RLEvent> ftl_events[6]; //Length equal to (#piko-elements - 1)
    static double angle_v=0;
    static double mk = 0.15; //Length between wagon midpoints

    if(t->getJointAngle(0,0) < -0.001 || t->getJointAngle(0,0) < -0.001)
    {
        t->setJointAngle(0,0,0.0);
    }
    //cout << "Performing action="<<action << endl;
    switch (action)
    {
        case 1: //Decrease vertical angle
                //myfile << "Decrease vertical angle" << endl;
                angle_v -= 0.05;
                if (angle_v < -1.3){ angle_v = -1.3; }
                t->setJointAngle(0,1,angle_v);
                
                ftl_events[0].push(RLEvent(t->getTravelledDistance()+mk,0.0,angle_v));
            break;
        case 2: //Increase vertical angle
            //myfile << "Increase vertical angle" << endl;
                angle_v += 0.05;
                if (angle_v > 1.3){ angle_v = 1.3; }
                t->setJointAngle(0,1,angle_v);
                
                ftl_events[0].push(RLEvent(t->getTravelledDistance()+mk,0.0,angle_v));
            break;
        case 3: //Move backwards
            //myfile << "Move backwards" << endl;
            speed+= -0.01;
            t->setMotorSpeedAll(speed);
            break;
        case 4: //Move forward
            //myfile << "Move forward" << endl;
            //Increase velocities by 10
            speed+= 0.01;
            t->setMotorSpeedAll(speed);
            break;
        case 5: //Reset
            //myfile << "Move backwards" << endl;
            for(int i=0; i< t->getNumCarts() -1 ; i++){
                while(ftl_events[i].size()>0){
                    ftl_events[i].pop();
                }
            }
            for (int i=0; i< t->getNumJoints(); i++)
            	angles[i]=0;

            chapter = 0;
            milestone = 0;
            activeJoints = 3;
            t->resetTrain();
            angle_v = 0.0;
            greedy = 0;
            expl = 0;
            break;
        default:
            t->setMotorSpeedAll(0);
            break;
    }

    //Extracts events from the queue at 'mk' intervals, propagating commands from the leader.
   for (int i=1; i < t->getNumCarts(); i++){ //For all the carts
	    if (ftl_events[i-1].empty()) // If the previous cart's event queue is empty, continue
            continue;
	    if (t->getTravelledDistance() >= ftl_events[i-1].front().dist){ 
        // If the train's traveled distance is greater or equal than the previous cart's distance
	        t->setJointAngle(i, 0, ftl_events[i-1].front().hor);    // Set the joint's H-angle to be the previous cart's first event's h-angle
	        t->setJointAngle(i, 1, ftl_events[i-1].front().vert);   // Set the joint's V-angle to be the previous cart's first event's v-angle
	        ftl_events[i-1].front().dist += mk;                     // Increment the previous cart's front  event's distance by "mk" 
	        
            if (i < t->getNumCarts()-1)                             // If there are more carts 
                ftl_events[i].push(ftl_events[i-1].front());        // Push the previous cart's events to the current cart's event queue
	        ftl_events[i-1].pop();                                  // Pop the previous cart's event queue
	        //cout << "POP<" << history << ">: " << i-1 << endl;
        }
    }
}

double RL::gaussianGenerator(double mu, double sigma)
{
    double x1, x2, w, y1, y2, r1, r2;
   // ofstream myfile;
    
   // myfile.open("randoms.txt", ios::out|ios::app);

         do {
                r1 = randDouble(0.0,1.0);
                r2 = randDouble(0.0,1.0);
                //cout << "r1=" << r1 << " r2=" << r2 <<endl;
                x1 = 2.0 * r1 - 1.0;
                x2 = 2.0 * r2 - 1.0;
                w = x1 * x1 + x2 * x2;
         } while ( w >= 1.0 );

         w = sqrt( (-2.0 * log( w ) ) / w );
         y1 = x1 * w;
         //y2 = x2 * w;
    y1 = sigma*y1 + mu;
    
   // myfile << y1 << endl;
   // myfile.close();
    return y1;
}

double RL::randDouble(double low, double high)
{
    double temp;

    /* swap low & high around if the user makes no sense */
    if (low > high)
    {
        temp = low;
        low = high;
        high = temp;
    }

    /* calculate the random number & return it */
    temp = (rand() / (static_cast<double>(RAND_MAX) + 1.0))
    * (high - low) + low;
    return temp;
}
void RL::dumpweights(int ep, int output)
{
	int k;
	cout << "dumpweigh_in" << endl;
	network->dumpWeights(ep, output);

	cout << "network_out" << endl;
}

int RL::signum(double a, double b)
// Returns 1 if they are different
{
	if( a == 0 && b == 0)
	{
		return 0;
	}else if( a < 0 && b < 0)
	{
		return 0;
	}else if(a > 0 && b > 0)
	{
		return 0;
	}else
		return 1;
}
double RL::vertical(Train *t)
{
    /*
	if( (t->getHeight(0) - t->getHeight(1)) > 0.2 )
		return 1;*/
	return t->getHeight(0) - t->getHeight(1);
}

void RL::getnOutputs()
{

    cout << "Capacity is: "<< network->getOutputs() << endl;
}
int RL::anglesReseted(Train *t)
{
	int ret=0;

	for( int i =0; i < t->getNumJoints() && ret == 0; i++)
	{
		if( abs(t->getJointAngle(i,1)) > 0.1)
			ret = 1;
		//cout << i << endl;
	}
//	cout << "reseting:"<<ret << endl;
	return ret;
}
