#include "Vent.h"

// Vent constructor without scaling
Vent::Vent(int t)
{
	scalex = 1;
	scaley = 1;
	scalez = 1;
	Vent(t, scalex, scaley, scalez);
}

//Vent constructor with scaling added
Vent::Vent(int t, dReal sx, dReal sy, dReal sz)
{
	scalex = sx;
	scaley = sy;
	scalez = sz;

	type = t;
	geom = dCreateSphere(0, 1);

	//set up collision geoms
	collisionGeoms[0] = dCreateBox(0, VENT_THICKNESS*scalex, scaley, scalez);
	collisionGeoms[1] = dCreateBox(0, VENT_THICKNESS*scalex, scaley, scalez);
	dGeomSetBody(collisionGeoms[0],0);
	dGeomSetBody(collisionGeoms[1],0);

	dGeomSetPosition(collisionGeoms[0],(-0.5+(VENT_THICKNESS/2))*scalex,0,0);
	dGeomSetPosition(collisionGeoms[1],(0.5-(VENT_THICKNESS/2))*scalex,0,0);

	if(type == 0 || type == 1){
		collisionGeoms[2] = dCreateBox(0, scalex, scaley, VENT_THICKNESS*scalez);
		dGeomSetBody(collisionGeoms[2],0);
		dGeomSetPosition(collisionGeoms[2],0,0,(0.5-(VENT_THICKNESS/2))*scalez);
	}

	if(type == 1 || type == 2){
		collisionGeoms[3] = dCreateBox(0, scalex, VENT_THICKNESS*scaley, scalez);
		dGeomSetBody(collisionGeoms[3],0);
		dGeomSetPosition(collisionGeoms[3],0,(-0.5+(VENT_THICKNESS/2))*scaley,0);
	}

		if(type == 2){
		collisionGeoms[2] = dCreateBox(0, scalex, VENT_THICKNESS*scaley, scalez);
		dGeomSetBody(collisionGeoms[2],0);
		dGeomSetPosition(collisionGeoms[2],0,(+0.5-(VENT_THICKNESS/2))*scaley,0);
	}
}

void Vent::addToViewer(Viewer *v)
{
	if (v==NULL)
		return;
	IAnimatedMesh* mesh;
	if(type == 0){
		mesh = v->getSceneManager()->getMesh("../media/SjaktmodulT.stl");
	}else if(type == 1){
		mesh = v->getSceneManager()->getMesh("../media/SjaktmodulL.stl");
	}else {
		type = 2;
		mesh = v->getSceneManager()->getMesh("../media/Sjaktmodul.stl");
	}

	node = v->getSceneManager()->addAnimatedMeshSceneNode(mesh);
	dGeomSetData(geom, node);
	node->setScale(vector3df(VENT_SCALE*scalex, VENT_SCALE*scaley, VENT_SCALE*scalez));
	dGeomSetBody(geom, 0);

	//Set drawing method and add geometry to the viewer for drawing
	node->setMaterialType(EMT_TRANSPARENT_ADD_COLOR);
	node->setMaterialTexture(0, v->getVideoDriver()->getTexture("../media/ground.ppm"));
	node->setMaterialFlag(video::EMF_LIGHTING, false);
	
	v->addObject(this->geom);
}

void Vent::setPosition(dReal x,dReal y, dReal z){
	//set position
	dGeomSetPosition(geom, x, y, z);
	dVector3 origo = {x,y,z,0};
	const dReal* R = dGeomGetRotation(geom);

	//rotate and position collision geometries
	dVector3 rel0,rel1,rel2,rel3;
	
	dVector3 o0 = {(-0.5+(VENT_THICKNESS/2))*scalex,0,0,0};
	dMultiply0(rel0,R,o0,3,3,1);
	dGeomSetPosition(collisionGeoms[0],origo[0]+rel0[0],origo[1]+rel0[1],origo[2]+rel0[2]);
	dGeomSetRotation(collisionGeoms[0], R);

	dVector3 o1 = {(+0.5-(VENT_THICKNESS/2))*scalex,0,0,0};
	dMultiply0(rel1,R,o1,3,3,1);
	dGeomSetPosition(collisionGeoms[1],origo[0]+rel1[0],origo[1]+rel1[1],origo[2]+rel1[2]);
	dGeomSetRotation(collisionGeoms[1], R);

	if(type == 0 || type == 1){
		dVector3 o2 = {0,0,(0.5-(VENT_THICKNESS/2))*scalez,0};
		dMultiply0(rel2,R,o2,3,3,1);
		dGeomSetPosition(collisionGeoms[2],origo[0]+rel2[0],origo[1]+rel2[1],origo[2]+rel2[2]);
		dGeomSetRotation(collisionGeoms[2], R);
	}

	if(type == 1 || type == 2){
		dVector3 o3 = {0,(-0.5+(VENT_THICKNESS/2))*scaley,0,0};
		dMultiply0(rel3,R,o3,3,3,1);
		dGeomSetPosition(collisionGeoms[3],origo[0]+rel3[0],origo[1]+rel3[1],origo[2]+rel3[2]);
		dGeomSetRotation(collisionGeoms[3], R);
	}

	if(type == 2){
		dVector3 o2 = {0,(+0.5-(VENT_THICKNESS/2))*scaley,0,0};
		dMultiply0(rel2,R,o2,3,3,1);
		dGeomSetPosition(collisionGeoms[2],origo[0]+rel2[0],origo[1]+rel2[1],origo[2]+rel2[2]);
		dGeomSetRotation(collisionGeoms[2], R);
	}
}

void Vent::setRotation(const dMatrix3 R){
	//set rotation 
	dGeomSetRotation(geom, R);
	const dReal* origo = dGeomGetPosition(geom);

	//rotate and position collision geometris
	dVector3 rel0,rel1,rel2,rel3;
	
	dVector3 o0 = {(-0.5+(VENT_THICKNESS/2))*scalex,0,0,0};
	dMultiply0(rel0,R,o0,3,3,1);
	dGeomSetPosition(collisionGeoms[0],origo[0]+rel0[0],origo[1]+rel0[1],origo[2]+rel0[2]);
	dGeomSetRotation(collisionGeoms[0], R);

	dVector3 o1 = {(+0.5-(VENT_THICKNESS/2))*scalex,0,0,0};
	dMultiply0(rel1,R,o1,3,3,1);
	dGeomSetPosition(collisionGeoms[1],origo[0]+rel1[0],origo[1]+rel1[1],origo[2]+rel1[2]);
	dGeomSetRotation(collisionGeoms[1], R);

	if(type == 0 || type == 1){
		dVector3 o2 = {0,0,(0.5-(VENT_THICKNESS/2))*scalez,0};
		dMultiply0(rel2,R,o2,3,3,1);
		dGeomSetPosition(collisionGeoms[2],origo[0]+rel2[0],origo[1]+rel2[1],origo[2]+rel2[2]);
		dGeomSetRotation(collisionGeoms[2], R);
	}

	if(type == 1 || type == 2){
		dVector3 o3 = {0,(-0.5+(VENT_THICKNESS/2))*scaley,0,0};
		dMultiply0(rel3,R,o3,3,3,1);
		dGeomSetPosition(collisionGeoms[3],origo[0]+rel3[0],origo[1]+rel3[1],origo[2]+rel3[2]);
		dGeomSetRotation(collisionGeoms[3], R);
	}

	if(type == 2){
		dVector3 o2 = {0,(+0.5-(VENT_THICKNESS/2))*scaley,0,0};
		dMultiply0(rel2,R,o2,3,3,1);
		dGeomSetPosition(collisionGeoms[2],origo[0]+rel2[0],origo[1]+rel2[1],origo[2]+rel2[2]);
		dGeomSetRotation(collisionGeoms[2], R);
	}
}

void Vent::addToSpace(dSpaceID space){
	//type of vent determines how many collision faces it has. I and L vents have 4 while T vents have 3.
	int lim = 4;
	if(type == 0) lim = 3;

	for(int i=0; i<lim; i++){
		dSpaceAdd(space, collisionGeoms[i]);
	}
}
