#include "Viewer.h"
#include "Object.h"
#include <iostream>

#include <irrlicht/ICameraSceneNode.h>

Viewer::Viewer(int width, int height) {
	device = irr::createDevice(irr::video::EDT_OPENGL, irr::core::dimension2d<irr::u32>(width, height), 16, false, false, false, &receiver);
	device->setWindowCaption(L"Slangerobot");
	
	driver = device->getVideoDriver();
	smgr = device->getSceneManager();
	guienv = device->getGUIEnvironment();
	
	camera = smgr->addCameraSceneNodeFPS(0, 50, 0.005);
	camera->setNearValue(0.001);
	camera->bindTargetAndRotation(true);

	device->getCursorControl()->setVisible(false);
	smgr->getActiveCamera()->setPosition(vector3df(2,2,2));
	smgr->getActiveCamera()->setRotation(vector3df(45,0,0));

	headCamera = smgr->addCameraSceneNodeFPS();
	headCamera->setNearValue(0.001);

	//smgr->setAmbientLight(video::SColorf(0.1,0.1,0.1,1));	
	//ILightSceneNode* light1 = smgr->addLightSceneNode( 0, core::vector3df(0,10,0), video::SColorf(0.9f,0.9f,0.9f), 1.0f, 1 );
	
	IAnimatedMesh* base = smgr->addHillPlaneMesh("Plane", dimension2d<f32>(3.0f,3.0f), dimension2d<u32>(100,100), 0, 0, dimension2d<f32>(0,0), dimension2d<f32>(250,250)); 
	ISceneNode* node = smgr->addAnimatedMeshSceneNode(base);
	node->setMaterialTexture(0, driver->getTexture("../media/rockwall.jpg")); //irrlicht/media/rockwall.bmp
	node->setMaterialFlag(video::EMF_LIGHTING, false);
	
	ISceneNode* node2 = smgr->addSkyDomeSceneNode(driver->getTexture("../media/skydome.jpg")); //irrlicht/media/skydome.jpg
	node2->setMaterialFlag(video::EMF_LIGHTING, false);
}

Viewer::~Viewer()
{
	device->closeDevice();
	device->drop();
}

void Viewer::updateScene() {
	for(int i = 0; i < (int)geoms.size(); i++) {
		ISceneNode* node = (ISceneNode*)dGeomGetData(geoms[i]);

		const dReal right2left[]={-1,0,0,0,0,0,1,0,0,-1,0,0};
		dVector3 position;
		dMultiply0(position, right2left, (dReal*)dGeomGetPosition(geoms[i]), 3, 3, 1);
		node->setPosition(vector3df(position[0], position[1], position[2]));		

		vector3df rot3;
		const dReal right2left2[]={-1,0,0,0,0,0,1,0,0,1,0,0};
		dQuaternion r, u;
		dRtoQ(right2left2, r);
		const dQuaternion s = {r[0], r[1], r[2], r[3]};
		dQuaternion result;
		dGeomGetQuaternion(geoms[i], result);
		const dQuaternion t = {result[0],result[1],result[2],result[3]};
		dQMultiply0(u,s,t);
		const quaternion q3(u[1],u[2],u[3],u[0]);

		q3.toEuler(rot3);
		vector3df degrees(-rot3.X*180/PI, -rot3.Y*180/PI, rot3.Z*180/PI);
		node->setRotation(degrees);	
	}
}

void Viewer::addObject(dGeomID g) {
	geoms.push_back(g);
}

IVideoDriver* Viewer::getVideoDriver() {
	return driver;
}

ISceneManager* Viewer::getSceneManager() {
	return smgr;
}

bool Viewer::draw() {
	if(device->run()) {
		updateScene();

		driver->beginScene(true, true, SColor(255,100,101,140));

		smgr->setActiveCamera(camera);
		driver->setViewPort(rect<s32>(0,0,800,600));
		smgr->drawAll();

		smgr->setActiveCamera(headCamera);
		driver->setViewPort(rect<s32>(0,0,250,250));
		smgr->drawAll();

		smgr->setActiveCamera(camera);

		guienv->drawAll();
		driver->endScene();

		return true;
	} else {
		device->drop();
		return false;
	}
}

void Viewer::positionCamera(double *position, double *direction)
{
	headCamera->setPosition(vector3df(-position[0], position[2], -position[1]));
	headCamera->setTarget(vector3df(-(position[0]+direction[0]), position[2]+direction[2], -(position[1]+direction[1])));
}
