#include <iostream>
#include "ANN.h"

using namespace std;

ANN::ANN(const vector<int>&layout, const vector<double>&weights)
{
	//Determine number of layout weights:
	unsigned int w = 0;
	for (unsigned int i=0;i<layout.size()-1;i++) {
		matrix_position.push_back(w);
		w += layout[i]*layout[i+1];
	}
	layout_weights = w;		

	//Determine number of hidden nodes:
	layout_nodes = 0;
	for (unsigned int i=1;i<layout.size()-1;i++) {
		layout_nodes += layout[i];
	}

//	cout << "Number of weights: " << weights.size() << endl;
//	cout << "Layout number of weights: " << layout_weights << endl;
//	cout << "Layout hidden nodes: " << layout_nodes << endl;

	this->layout = layout;
	this->weights = weights;
}


double ANN::getWeight(unsigned int layer, unsigned int from, unsigned int to)
{
	unsigned int start = matrix_position[layer];
	unsigned int cols = layout[layer+1];
//	cout << "weights.size(): " << weights.size() << "start: " << start << " cols: " << cols << " from: " << from << " to: " << to << " layer: " << layer <<  endl;
	return weights[start + from*cols + to];
}


void ANN::getOutput(const vector<double> &inputs, vector<double> &outputs)
{
	vector<double> in = inputs;

	//For each hidden layer:
	for (unsigned int l=1;l<layout.size()-1;l++) {

		//Vector to hold node results:
		vector<double> node_res;

		//For each node in layer:
		for (int n=0;n<layout[l];n++) {

			//Find sum of all inputs:
			double sum = 0;
			for (unsigned int i=0;i<in.size();i++) {
				sum += in[i]*getWeight(l-1, i, n);
			}

			//TODO: Apply threshold function:
			if (sum > 0) sum = 1;
			
			//Store in node_res:
			node_res.push_back(sum);
		}
		
		//Set this layers output to the next layer input:
		in = node_res;
	}

	//For each output node:
	for (int n=0;n<layout.back();n++) {
		
		//Find sum of all inputs:
		double sum = 0;
		for (unsigned int i=0;i<in.size();i++) {
			sum += in[i]*getWeight(layout.size()-2, i, n);
		}

		//Store in output:
		outputs.push_back(sum);
	}
	
}
