#include <iostream>
using namespace std;
#include <cstdlib>
#include "EAProblem.h"
#include "EAScaling.h"


EAProblem::EAProblem()
{
	generation = 0;

	//Set default values:
	num_adults = 10;
	num_childs = 10;
	num_elites = 1;

	p_crossover = 0.0;
	p_mutation = 0.01;
	
	scaling = new EAUnscaled;
}

EAProblem::~EAProblem()
{
	if (scaling) delete scaling;
}

unsigned int EAProblem::wheel_spin(const vector<double> &wheel)
{
	unsigned int i;
	double value = (double)rand()/RAND_MAX;
	for (i=0; i<wheel.size() ; i++)
		if ( (value-=wheel[i]) <= 0) return i;
	return wheel.size() - 1;
}

void EAProblem::run(long gens)
{
	if (generation == 0) {
		population.clear();
		pop_init();
	}

	for (int g=0; g<gens; g++) {

		//Calculate fitnesses for the entire population:
		for (unsigned int i=0; i<population.size(); i++) population[i]->setFitness(calc_fitness(population[i]));

		//Sort population:
		population.sort();

		//Print the best genomes:
		cout << "---------------------------" << endl;
		cout << "Generation " << g << endl;
		cout << "---------------------------" << endl;
		for (unsigned int i=0; i<population.size(); i++) { 
			cout << i << ". " << population[i]->getFitness() << endl;
		}
		
		//Generate adult pool:
		EAPopulation *adults = new EAPopulation(population);
		adults->shrinkTo(num_adults);

		//Make a wheel of fitnesses for adult pool:
		vector<double> wheel;
		for (unsigned int i=0; i<adults->size() ; i++) {
			wheel.push_back(adults->at(i)->getFitness());
		}

		//Scale the wheel with selected scaling:
		scaling->scale(wheel);
		EAScale::normalize(wheel);

		//Shrink population pool to contain elites only:
		population.shrinkTo(num_elites);

		//Add childs to population pool:	
		for (unsigned int c = 0 ; c < num_childs; c++) {

			//Select one parent:
			unsigned int parent0 = wheel_spin(wheel);
			//cout << "Parent0 = " << parent0 << endl;
			//Make a new child:
			EAIndividual *child;
	
			//Sexual / asexual ?
			if (rand()/RAND_MAX < p_crossover) {
				unsigned int parent1 = wheel_spin(wheel);
				//cout << "Parent1 = " << parent1 << endl;
				
				child = adults->at(parent0)->crossover(adults->at(parent1));
			}else {
				child = adults->at(parent0);
			}
		
			//Mutate
			child->mutate(p_mutation);
	
			//Add the newborn child to population:
			population.push(child);
		}

		generation++;
	}
}
