
#include "Snake.h"

#include <cmath>
#include <cstdio>
#include <cstring>
#include <iostream>

#include "ANN.h"
#include "SnakeIndividual.h"


#include "../simulator/Train.h"

using namespace std;

Snake::Snake() {

	timesteps = 10 / 0.05;

	int ann[] = {2, 10, 2};
	annLayout = vector<int>(ann, ann+3);

	ANN a(annLayout, vector<double>());
	weights = a.layoutWeights();

	visual = true;
	simulator = new Simulator(-1.5, 0, 0.2, 0.05, visual);
}

double Snake::calc_fitness(EAIndividual *individual)
{
	// Recreate simulator if visualization setting have changed
	if (visual == true && simulator->getVisual() == false) {
		delete(simulator);
		simulator = new Simulator(-1.5, 0, 0.2, 0.05, true);
	}
	else if (visual == false && simulator->getVisual() == true) {
		delete(simulator);
		simulator = new Simulator(-1.5, 0, 0.2, 0.05, false);
	}

	SnakeIndividual *ind = dynamic_cast<SnakeIndividual*>(individual);

	double retval = 0;
	Train *t = simulator->getTrain();
	t->resetTrain();

	for (unsigned int i=0; i<100; i++) 
	{
		//For all modules: 
		for (int m =0; m < t->getNumCarts(); m++) {

			//Get input:	
			vector<double> input;
			input.push_back(t->getJointAngle(m, 0));
			input.push_back(t->getJointAngle(m, 1));

			//Find output:p
			ANN ann(annLayout, ind->wagon_list()[m]->double_list());
			vector<double> output;

			ann.getOutput(input, output);

			//Set output:
			t->setJointAngle(m, 0, output[0]);
			t->setJointAngle(m, 1, output[1]);
			t->setMotorSpeed(m, 10);
		}
		simulator->step();
	}
	
	retval = t->getTravelledDistance();
	
	return retval;
}

void Snake::pop_init(void)
{
	//Initialize population.
	for (unsigned int i=0;i<getNumAdults();i++){
		population.push(new SnakeIndividual(6, weights));
	}
}
