#include "TSP.h"
#include <cmath>
#include <cstdio>
#include <cstring>
#include <iostream>
using namespace std;

#include "EAUniqueInteger.h"

TSP::TSP() {

}

TSP::TSP(const char *file)
{
	TSP();
	LoadFromFile(file);
}

bool TSP::LoadFromFile(const char *file)
{
	//Clear all points:
	tsp_point.clear();

	// Open file:
	FILE *f = fopen(file, "r");
	if (f==NULL) return false;
	
	int size = 0;
	char buffer[1024];
	
	// Get dimension, scan to coord section:
	while (!feof(f)) {
		fgets(buffer, 1024, f);
		sscanf(buffer, "DIMENSION: %i\n", &size);
		if (strcmp(buffer, "NODE_COORD_SECTION\n")==0) break;
	}

	printf("Dim: %i points.\n", size);
		
	// Read points:
	int n;
	point_t p;
	while (!feof(f)) {
		fscanf(f, "%i %f %f\n", &n, &p.x, &p.y);
	 	tsp_point.push_back(p);
	}
	
	//Close file
	fclose(f);

	return true;
}

void TSP::graphBest(void)
{
	EAUniqueInteger *g = reinterpret_cast<EAUniqueInteger*>(population.getBest());
	vector<int> *ints = g->integer_list();

	for (unsigned int i=0; i<ints->size() ; i++) {
		cout << tsp_point[ints->at(i)].x << "\t" << tsp_point[ints->at(i)].y << endl;
	}
}

double TSP::calc_fitness(EAIndividual *individual)
{
	//Calculate fitness:
	EAUniqueInteger *g = reinterpret_cast<EAUniqueInteger*>(individual);
	vector<int> *ints = g->integer_list();

	unsigned int j;
	double dist = 0;
	for (j=0; j < ints->size()-1; j++) {
		dist += sqrt(pow((tsp_point[ints->at(j)].x - tsp_point[ints->at(j+1)].x),2) + pow((tsp_point[ints->at(j)].y - tsp_point[ints->at(j+1)].y),2));
	}

	//Add distance from last city to first:
	dist += sqrt(pow((tsp_point[ints->at(0)].x - tsp_point[ints->at(j)].x),2) + pow((tsp_point[ints->at(0)].y - tsp_point[ints->at(j)].y),2));

	//Return fitness value:
	return (1/dist);
}

void TSP::pop_init(void)
{
	//Initialize population.
	for (unsigned int i=0;i<getNumAdults();i++){
		population.push(new EAUniqueInteger(tsp_point.size()));
	}
}
