#include "Controller.h"
#include "EventReceiver.h"

#include <iostream>
#include <queue>

//Basic controller (W,A,S,D,Q,E) setting speed/angle directly.
void controller(Train *t, Viewer *v)
{
	if (v->receiver.IsKeyDown(irr::KEY_KEY_W))
		t->setMotorSpeedAll(10);
	else if (v->receiver.IsKeyDown(irr::KEY_KEY_S))
		t->setMotorSpeedAll(-10);
	else
		t->setMotorSpeedAll(0);

	if (v->receiver.IsKeyDown(irr::KEY_KEY_A))
		t->setJointAngle(0, 0, 0.5);
	else if (v->receiver.IsKeyDown(irr::KEY_KEY_D))
		t->setJointAngle(0, 0, -0.5);
	else
		t->setJointAngle(0, 0, 0);

	if (v->receiver.IsKeyDown(irr::KEY_KEY_Q)) {
		t->setJointAngle(0, 1, 0.5);
		t->setJointAngle(1, 1, 0.5);
		for (int i = 2; i < t->getNumCarts(); i++) {
			t->setJointAngle(i, 1, 0);
		}
	}
	else if (v->receiver.IsKeyDown(irr::KEY_KEY_E)) {
		t->setJointAngle(0, 1, -0.5);
		t->setJointAngle(1, 1, -0.5);
		for (int i = 2; i < t->getNumCarts(); i++) {
			t->setJointAngle(i, 1, 0);
		}
	}
	else {
		for (int i = 0; i < t->getNumCarts(); i++) {
			t->setJointAngle(i, 1, 0);
		}
	}

	if (v->receiver.IsKeyDown(irr::KEY_KEY_R))
		t->resetTrain();
}

//Controller with "Follow the leader"-algorithm
void controllerFTL(Train *t, Viewer *v){

  static queue<Event> ftl_events[5]; //Length equal to (#piko-elements - 1)
  static bool isLeft=false, isRight=false, isUp=false, isDown=false;
  static double angle_h, angle_v;
  static double mk = 0.15; //Length between wagon midpoints

  // Set angles as defined in Train for each iteration
  
  for (int i=0; i < t->getNumJoints(); i++){
    t->setJointAngle(i,0,t->getJointAngleSetPoint(i,0)); //Update horisontal reference
    t->setJointAngle(i,1,t->getJointAngleSetPoint(i,1)); //Update vertical reference
  }

  if (v->receiver.IsKeyDown(irr::KEY_KEY_A)){
    if (!isLeft){
      isLeft = true;
      angle_h += 0.1;
      //      cout << "Angle inc " << angle_h << endl;
      if (angle_h > 0.7){ angle_h = 0.7; }
      t->setJointAngle(0,0,angle_h);
      ftl_events[0].push(Event(t->getTravelledDistance()+mk,angle_h,angle_v));
    }
  }
  else{ isLeft = false; }

  if (v->receiver.IsKeyDown(irr::KEY_KEY_D)){
    if (!isRight){
      isRight = true;
      angle_h -= 0.1; //Decreasing lifts PIKO
      //      cout << "Angle dec " << angle_h <<endl;
      if (angle_h < -0.7){ angle_h = -0.7; }
      t->setJointAngle(0,0,angle_h);
      ftl_events[0].push(Event(t->getTravelledDistance()+mk,angle_h,angle_v));
    }
  }
  else{ isRight = false; }

  if (v->receiver.IsKeyDown(irr::KEY_KEY_Q)){
    if (!isUp){
      isUp = true;
      angle_v += 0.1; //Increasing turns PIKO downwards
      //      cout << "Angle_v inc " << angle_v <<endl;
      if (angle_v > 0.5){ angle_v = 0.5; }
      t->setJointAngle(0,1,angle_v);
      ftl_events[0].push(Event(t->getTravelledDistance()+mk,angle_h,angle_v));
    }
  }
  else{ isUp = false; }


  if (v->receiver.IsKeyDown(irr::KEY_KEY_E)){
    if (!isDown){
      isDown = true;
      angle_v -= 0.1;
      //cout << "Angle_v dec " << angle_v <<endl;
      if (angle_v < -0.5){ angle_v = -0.5; }
      t->setJointAngle(0,1,angle_v);
      ftl_events[0].push(Event(t->getTravelledDistance()+mk,angle_h,angle_v));
    }
  }
  else{ isDown = false; }


  if (v->receiver.IsKeyDown(irr::KEY_KEY_W))
    t->setMotorSpeedAll(10);
  else if (v->receiver.IsKeyDown(irr::KEY_KEY_S))
    t->setMotorSpeedAll(-10);
  else
    t->setMotorSpeedAll(0);

  //Extracts events from the queue at 'mk' intervals, propagating commands from the leader.
    for (int i=1; i < t->getNumCarts(); i++){
      if (ftl_events[i-1].empty()) continue;
      if (t->getTravelledDistance() >= ftl_events[i-1].front().dist){
	t->setJointAngle(i, 0, ftl_events[i-1].front().hor);
	t->setJointAngle(i, 1, ftl_events[i-1].front().vert);
	ftl_events[i-1].front().dist += mk;
	if (i < t->getNumCarts()-1) ftl_events[i].push(ftl_events[i-1].front());
	ftl_events[i-1].pop();
      }
    }
    
}

void controllerUp(Train *t, Viewer *v){
	static int iteration = 0;

	t->setMotorSpeedAll(0);
	for (int i = 0; i < t->getNumJoints(); i++)
		t->setJointAngle(i, 0, 0);

	int joint = (iteration/100<=t->getNumJoints())?iteration/100:0;
	if (iteration < 700) {
		for (int i = 0; i < joint; i++)
			t->setJointAngle(i, 1, 0.8);
		for (int i = joint; i < t->getNumJoints(); i++)
			t->setJointAngle(i, 1, 0);
	} else if ((iteration > 50) && (iteration < 100)) {
		for (int i = 0; i < t->getNumJoints(); i++)
			t->setJointAngle(i, 1, 0.5);
	} else if ((iteration > 100) && (iteration < 200)) {
		for (int i = 0; i < t->getNumJoints(); i++)
			t->setJointAngle(i, 1, 0.5);
	} else if (iteration > 200) {
	}

	iteration++;
}
