#ifndef TRAIN_H
#define TRAIN_H

#include "Bracket.h"
#include "PIKo_Module.h"

#include <vector>

#define CARTS_DEFAULT 	6 	//Default number of train modules.
#define CART_SPACE 0.106 // Distance between train modules [m]
#define ANGLE_CONSTR 0.6 // Angle constraints on joints //1.22 (70 degrees)

#define CHASS_W 0.120
#define CHASS_H 0.110
#define CHASS_D 0.130
#define CHASS_M 1.252 

#define WHEEL_R 0.025
#define WHEEL_W 0.006
#define WHEEL_M 0.05 //Unknown variable

#define MAX_MOTOR_TORQUE 0.01 //TODO: Check
#define MAX_JOINT_TORQUE 10 //TODO: Check
#define MAX_MOTOR_VELOCITY 0.5 //TODO: Check
#define MAX_JOINT_VELOCITY 0.28 //TODO: Check

#define BRACK_W 0.01f
#define BRACK_H 0.05f
#define BRACK_D 0.05f
#define BRACK_M 0.05f

#define FORK_LENGTH 0.05

class Train : public Object
{
	protected:
		double distance;
		dReal lastPosition[3];
		dReal lastError;
		dReal integral;

		int num_carts;			//Number of modules
		vector<PIKo_Module*> module;	//Modules
		vector<Bracket*> bracket;	//Brackets between the joints
		vector<dJointID> vertjoint;	//Vertical joints
		vector<dJointID> horjoint;	//Horisontal joints
		vector<dReal> jointAngleSetPointVert;
		vector<dReal> jointAngleSetPointHor;
		dGeomID noseRay;
		double wheelDistance;
		dReal initial_x, initial_y, initial_z;

	public:
		Train(dWorldID);
		Train(dWorldID, int carts);

		int getNumCarts(void) {
			return num_carts;
		};

		int getNumJoints(void) {
			return num_carts - 1;
		};

		void setPosition(dReal x, dReal y, dReal z);
		void addToSpace(dSpaceID space);
		void addToViewer(Viewer *v);

		PIKo_Module *getFirstModule(void) {
			return module[0];
		}

		void resetTrain(void);

		// Control
		void setMotorSpeed(int moduleNumber, dReal speed);
		void setMotorSpeedAll(dReal speed);
		void setMotorTorque(int moduleNumber, dReal speed);
		void setMotorTorqueAll(dReal speed);

		void setJointAngle(int jointNumber, int direction, dReal angle);
		void setJointAngleHorizontal(dReal angle);
		void setJointAngleVertical(dReal angle);
		void setJointTorque(int jointNumber, int direction, dReal torque);
		void setJointTorqueHorizontal(dReal torque);
		void setJointTorqueVertical(dReal torque);
		void setJointVelocity(int jointNumber, int direction, dReal velocity);

		// Sensor
		double getDistance(dGeomID space, dReal pitch, dReal yaw);
		double getDistanceNose(dGeomID space);
		double getSonar(dGeomID space, double openingAngle);
		void getLaserScanner1D(dGeomID space, int points, double openingAngle, double* result);
		void getTimeOfFlightCam(dGeomID space, int verticalPoints, int horizontalPoints, double verticalOpeningAngle, double horizontalOpeningAngle, double **result);
		void getIMU(double **position, double **direction, double **linearVelocity, double **angularVelocity, double **linearAcc, double **angularAcc);
		double getJointAngle(int jointNumber, int direction);
		double getJointAngleSetPoint(int jointNumber, int direction);
		double getTravelledDistance(void);
		void resetDistance(void);
		double getWheelDistance(double timestep);
		void getJointForces(int jointNumber, double *up, double *down, double *left, double *right);
		double getHeigth(int moduleNumber);
};

#endif
