#include "VentStruct.h"

VentStruct::VentStruct(){
	geom = dCreateSphere(0, 1);					//used for the purposes of defining the structures pos and rot
	dGeomSetBody(geom, 0);									

	dMatrix3 R1, R2, R3;

	//Create each separate vent and set up their relative position and rotation
	vents[0] = new Vent(2,0.3,0.3,0.3);
	relPos[0][0] = 0;	relPos[0][1] = 0;	relPos[0][2] = 0; relPos[0][3] = 0;
	dRFromAxisAndAngle(relRot[0],0,1,0,90*PI/180);

	vents[1] = new Vent(0,0.3,0.3,0.3);
	relPos[1][0] = -0.3/2-0.3/2;	relPos[1][1] = 0;	relPos[1][2] = 0; relPos[1][3] = 0;
	dRFromAxisAndAngle(R1,0,0,1,90*PI/180);
	dRFromAxisAndAngle(R2,1,0,0,90*PI/180);
	dMultiply0(relRot[1],R2,R1,3,3,3);

	vents[2] = new Vent(2,0.3,0.3,0.6);
	relPos[2][0] = -0.3/2-0.3-0.6/2;	relPos[2][1] = 0;	relPos[2][2] = 0; relPos[2][3] = 0;
	dRFromAxisAndAngle(relRot[2],0,1,0,90*PI/180);

	vents[3] = new Vent(1,0.3,0.3,0.3);
	relPos[3][0] = -0.3/2-0.3-0.6-0.3/2;	relPos[3][1] = 0;	relPos[3][2] = 0; relPos[3][3] = 0;
	dRFromAxisAndAngle(relRot[3],0,1,0,-90*PI/180);

	vents[4] = new Vent(2,0.3,0.3,0.6);
	relPos[4][0] = -0.3/2-0.3-0.6/2;	relPos[4][1] = 0.3/2+0.6+0.3/2;	relPos[4][2] = 0; relPos[4][3] = 0;
	dRFromAxisAndAngle(relRot[4],0,1,0,90*PI/180);

	vents[5] = new Vent(2,0.3,0.3,0.6);
	relPos[5][0] = -0.3/2-0.3/2;	relPos[5][1] = 0.3/2+0.6/2;	relPos[5][2] = 0; relPos[5][3] = 0;
	dRFromAxisAndAngle(relRot[5],1,0,0,90*PI/180);

	vents[6] = new Vent(2,0.3,0.3,0.6);
	relPos[6][0] = -0.3/2-0.3-0.6-0.3/2;	relPos[6][1] = 0.3/2+0.6/2;	relPos[6][2] = 0; relPos[6][3] = 0;
	dRFromAxisAndAngle(relRot[6],1,0,0,89*PI/180);

	vents[7] = new Vent(1,0.3,0.3,0.3);
	relPos[7][0] = -0.3/2-0.3/2;	relPos[7][1] = 0.3/2+0.6+0.3/2;	relPos[7][2] = 0; relPos[7][3] = 0;
	dRFromAxisAndAngle(R1,1,0,0,-90*PI/180);
	dRFromAxisAndAngle(R2,0,1,0,90*PI/180);
	dMultiply0(relRot[7],R2,R1,3,3,3);

	vents[8] = new Vent(1,0.3,0.3,0.3);
	relPos[8][0] = -0.3/2-0.3-0.6-0.3/2;	relPos[8][1] = 0.3/2+0.6+0.3/2;	relPos[8][2] = 0; relPos[8][3] = 0;
	dRFromAxisAndAngle(relRot[8],0,1,0,0*PI/180);
	dRFromAxisAndAngle(R1,1,0,0,-90*PI/180);
	dRFromAxisAndAngle(R2,0,1,0,-90*PI/180);
	dMultiply0(relRot[8],R2,R1,3,3,3);
}

void VentStruct::setPosition(dReal x,dReal y, dReal z){
	dGeomSetPosition(geom, x, y, z);
	dVector3 origo = {x,y,z,0};
	const dReal* R = dGeomGetRotation(geom);

	for(int i=0; i<NUM_PARTS; i++){
		dVector3 worldRelPos;				//relative position in the world space
		dMatrix3 worldRelRot;				//rotation in the world space

		dMultiply0(worldRelPos,R,relPos[i],3,3,1);
		vents[i]->setPosition(origo[0]+worldRelPos[0],origo[1]+worldRelPos[1],origo[2]+worldRelPos[2]);
		
		dMultiply0(worldRelRot,R,relRot[i],3,3,3);
		vents[i]->setRotation(worldRelRot);
	}
}
void VentStruct::setRotation(const dMatrix3 R){
	dGeomSetRotation(geom, R);
	const dReal* origo = dGeomGetPosition(geom);

	for(int i=0; i<NUM_PARTS; i++){
		dVector3 worldRelPos;				//relative position in the world space
		dMatrix3 worldRelRot;				//rotation in the world space

		dMultiply0(worldRelPos,R,relPos[i],3,3,1);
		vents[i]->setPosition(origo[0]+worldRelPos[0],origo[1]+worldRelPos[1],origo[2]+worldRelPos[2]);
		
		dMultiply0(worldRelRot,R,relRot[i],3,3,3);
		vents[i]->setRotation(worldRelRot);
	}
}

void VentStruct::addToSpace(dSpaceID space){
	for(int i=0; i<NUM_PARTS; i++){
		vents[i]->addToSpace(space);
	}
}

void VentStruct::addToViewer(Viewer *v){
	if (v==NULL)
		return;
	for(int i=0; i<NUM_PARTS; i++){
		vents[i]->addToViewer(v);
	}
}
