#include "Wheel.h"

Wheel::Wheel(dWorldID world)
{
	dReal radius = 1.0f;
	dReal width = 1.0f;
	dReal mass = 1.0f;
	Wheel(world, radius, width, mass);
}

Wheel::Wheel(dWorldID world, dReal radius, dReal width, dReal mass)
{
	this->radius = radius;
	this->width = width;
	this->mass = mass;

	body = dBodyCreate(world);
	geom = dCreateSphere(0, radius);

	dMass m;
	dMassSetSphere(&m, 1.0f, radius);
	dMassAdjust(&m, mass);
	dBodySetMass(body, &m);

	dGeomSetBody(geom, body);

	dBodySetFiniteRotationMode(body, 1); // Try to avoid wheels turning along suspension axis
}

dReal Wheel::getRadius()
{
	return radius;
}

dReal Wheel::getWidth()
{
	return width;
}

void Wheel::addToViewer(Viewer *v)
{
	if (v==NULL)
		return;
	IAnimatedMesh* mesh = v->getSceneManager()->getMesh("../media/hjul2.stl");
	IAnimatedMeshSceneNode* node = v->getSceneManager()->addAnimatedMeshSceneNode(mesh);
	dGeomSetData(geom, node);
	
	dReal* position = (dReal*)dGeomGetPosition(getGeom());
	node->setPosition(vector3df((irr::f32)position[0], (irr::f32)position[1], (irr::f32)position[2]));
	node->setScale(vector3df(width / 10, width / 10, width / 10));
	node->setRotation(vector3df(90, 0, 0));

	node->setMaterialTexture(0, v->getVideoDriver()->getTexture("../media/checkered.ppm"));
	node->setMaterialFlag(video::EMF_LIGHTING, false);
	
	v->addObject(this->geom);
}
